﻿-------------------- Vars -------------------- 

AddOn = LibStub("AceAddon-3.0"):NewAddon("TalentProfiles", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0");

local const__numTalentCols = 3 -- TODO: Get this from the API

-------------------- Helpers -------------------- 

-- Prints all the key value pairs in the given table (See python's dir() function)
local function dir(t)
    for k, v in pairs(t) do
        print(k, v)
    end
end

local function CompactList(list)
    local l = list

end

-- Returns the length of the given table
local function GetTableLength(t)
    local count = 0
    for k, v in pairs(t) do
        count = count + 1
    end
    return count
end

-- Ensure the database is valid
local function VerifyDB()
    if TalentProfilesDB == nil then TalentProfilesDB = {} end
    if TalentProfilesDB.specs == nil then TalentProfilesDB.specs = {} end
    for i = 1, GetNumSpecializations() do
        if TalentProfilesDB.specs[i] == nil then TalentProfilesDB.specs[i] = {} end
        if TalentProfilesDB.specs[i].activeIndex == nil then TalentProfilesDB.specs[i].activeIndex = 0 end
        if TalentProfilesDB.specs[i].profiles == nil then TalentProfilesDB.specs[i].profiles = {} end
    end
end

-- Returns the talent info for each talent the user currently has available
local function GetTalentInfos()
    local talentInfos = {}
    local k = 1
    for i = 1, GetMaxTalentTier() do
        for j = 1, const__numTalentCols do
            local talentID, name, texture, selected, available, spellid, tier, column = GetTalentInfo(i, j, GetActiveSpecGroup())
            talentInfos[k] = { }
            talentInfos[k].talentID = talentID
            talentInfos[k].name = name
            talentInfos[k].texture = texture
            talentInfos[k].selected = selected
            talentInfos[k].available = available
            talentInfos[k].spellid = spellid
            talentInfos[k].tier = tier
            talentInfos[k].column = column
            k = k + 1
        end
    end
    return talentInfos
end

-- Adds a new profile to the database
local function AddProfile(name)
    -- Ensure the database is ready
    VerifyDB()
    -- Get basic info
    local talentInfos = GetTalentInfos()
    local profile = {}
    profile.name = name
    profile.talents = {}
    -- Get the currently selected talents
    local i = 1
    for k, v in pairs(talentInfos) do
        if v.selected == true then
            profile.talents[i] = v.talentID
            i = i + 1
        end
    end
    -- Make sure the data is valid
    if i > 8 then
        AddOn:Print("Error: Too many talents selected")
    end
    -- Save the profile to the database
    table.insert(TalentProfilesDB.specs[GetSpecialization()].profiles, profile)
    -- Rebuild the frame with the new data
    BuildFrame()
    -- Inform the user a profile was added
    AddOn:Print("Added a new profile: '" .. profile.name .. "'")
end

-- Saves the current talent configuration to the current profile
local function SaveProfile(index)
    -- Don't activate the placeholder profile
    if index ~= "new" then
        -- Get the profile, checking for errors on the way
        print('loading profile index = ', index)
        local profile = TalentProfilesDB.specs[GetSpecialization()].profiles[index]
        if profile == nil then
            AddOn:Print("Unable to load the selected profile")
            return
        end
        -- Update the selected talents
        local talentInfos = GetTalentInfos()
        local i = 1
        for k, v in pairs(talentInfos) do
            if v.selected == true then
                profile.talents[i] = v.talentID
                i = i + 1
            end
        end
        -- Inform the user a profile was activated
        AddOn:Print("Saved profile: '" .. profile.name .. "'")
    end
end

local function PopupHandler_AddProfile(sender)
    AddProfile(sender.editBox:GetText())
end

-- Remove a profile from the database
local function RemoveProfile(index)
    local key = nil
    local i = 1
    for k, v in pairs(TalentProfilesDB.specs[GetSpecialization()].profiles) do
        if i == index then
            key = k
            print(key)
        end
        i = i + 1
    end
    -- Cache the name
    local name = TalentProfilesDB.specs[GetSpecialization()].profiles[key].name
    -- Remove the profile
    table.remove(TalentProfilesDB.specs[GetSpecialization()].profiles, key)
    BuildFrame()
    -- Inform the user a profile was removed
    AddOn:Print("Removed a profile: '" .. name .. "'")
end

function PopupHandler_RemoveProfile(sender)
    RemoveProfile(TalentProfiles_profilesDropDown.selectedID)
end

-- Dialogue that enables the user to name a new profile
StaticPopupDialogs["TALENTPROFILES_ADD_PROFILE"] = {
    text = "Enter Profile Name:",
    button1 = "Save",
    button2 = "Cancel",
    OnAccept = PopupHandler_AddProfile,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    preferredIndex = 3,
    hasEditBox = true,
}
function StaticPopupShow_Add()
    StaticPopup_Show("TALENTPROFILES_ADD_PROFILE")
end

-- Dialogue that enables the user to confirm the removal of a profile
StaticPopupDialogs["TALENTPROFILES_REMOVE_PROFILE"] = {
    text = "Do you want to remove the profile '%s'?",
    button1 = "Yes",
    button2 = "No",
    OnAccept = PopupHandler_RemoveProfile,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    preferredIndex = 3,
}
function StaticPopupShow_Remove()
    local index = TalentProfiles_profilesDropDown.selectedID
    local name = TalentProfilesDB.specs[GetSpecialization()].profiles[index].name
    StaticPopup_Show("TALENTPROFILES_REMOVE_PROFILE", name)
end

-- Activates the profile at the given index
local function ActivateProfile(index)
    -- Don't activate the placeholder profile
    if index ~= "new" then
        -- Get the profile, checking for errors on the way
        local profile = TalentProfilesDB.specs[GetSpecialization()].profiles[index]
        if profile == nil or profile.talents == nil then
            AddOn:Print("Unable to load talent configuration for the selected profile")
            return
        end
        for i = 1, GetMaxTalentTier() do
            LearnTalent(profile.talents[i])
        end
        -- Save the index of the active profile
        TalentProfilesDB.specs[GetSpecialization()].activeIndex = index
        -- Inform the user a profile was activated
        AddOn:Print("Activated profile: '" .. profile.name .. "'")
    end
end

-------------------- TalentProfiles Event Handlers --------------------

-- Fired when the "Activate" button is clicked
function Handler_ActivateProfile(sender)
    VerifyDB()
    -- Check if any profiles exists
    if GetTableLength(TalentProfilesDB.specs[GetSpecialization()].profiles) == 0 then
        return
    end
    -- Activate the profile
    ActivateProfile(TalentProfiles_profilesDropDown.selectedID)
end

-- Fired when the "Save" button is clicked
function Handler_SaveProfile(sender)
    SaveProfile(TalentProfiles_profilesDropDown.selectedID)
end

-- Fired when the "Remove" button is clicked
function Handler_RemoveProfile(sender)
    VerifyDB()
    -- Check if any profiles exists
    if GetTableLength(TalentProfilesDB.specs[GetSpecialization()].profiles) == 0 then
        return
    end
    StaticPopupShow_Remove()
end

-------------------- Dropdown functions --------------------

function ProfilesDropDown_OnClick(sender)
    UIDropDownMenu_SetSelectedID(TalentProfiles_profilesDropDown, sender:GetID())
end

function ProfilesDropDown_OnClick_NewProfile(sender)
    StaticPopupShow_Add()
end

function ProfilesDropDown_Initialise(sender, level)
    VerifyDB()
    local items = TalentProfilesDB.specs[GetSpecialization()].profiles
    local i = 1
    for k, v in pairs(items) do
        local info = UIDropDownMenu_CreateInfo()
        info.text = v.name
        info.value = i
        info.func = ProfilesDropDown_OnClick
        UIDropDownMenu_AddButton(info, level)
        if i == TalentProfilesDB.specs[GetSpecialization()].activeIndex then
            info.checked = true
        else
            info.checked = false
        end
        i = i + 1
    end
    -- Add the new profile item
    local info = UIDropDownMenu_CreateInfo()
    info.text = "Add new profile"
    info.value = "new"
    info.func = ProfilesDropDown_OnClick_NewProfile
    info.rgb = { 1.0, 0.0, 1.0, 1.0 }
    UIDropDownMenu_AddButton(info, level)
end

function BuildFrame()
    -- Set up main frame, if it doesnt already exist
    local mainFrame = TalentProfiles_main
    if TalentProfiles_main == nil then
        mainFrame = CreateFrame("Frame", "TalentProfiles_main", PlayerTalentFrame)
        mainFrame:SetPoint("TOPLEFT", PlayerTalentFrame, "TOPLEFT", 50, -30)
        mainFrame:SetSize(400, 50)
    end

    -- Set up profiles dropdown, if it doesnt already exist
    local dropdown = TalentProfiles_profilesDropDown
    if TalentProfiles_profilesDropDown == nil then
        dropdown = CreateFrame("Button", "TalentProfiles_profilesDropDown", TalentProfiles_main, "UIDropDownMenuTemplate")
        UIDropDownMenu_SetWidth(dropdown, 125)
        UIDropDownMenu_SetButtonWidth(dropdown, 124)
    end
    -- Repopulate the dropdown, even if it already exists
    UIDropDownMenu_Initialize(dropdown, ProfilesDropDown_Initialise)
    UIDropDownMenu_SetSelectedID(dropdown, 1) -- TODO: select the currently active profile
    UIDropDownMenu_JustifyText(dropdown, "LEFT")
    dropdown:SetPoint("TOPLEFT", TalentProfiles_main, "TOPLEFT", 0, 0)
    dropdown:Show()

    -- Set up the action buttons
    local btnApply = TalentProfiles_btnApply
    if TalentProfiles_btnApply == nil then
        btnApply = CreateFrame("Button", "TalentProfiles_btnApply", TalentProfiles_main, "UIPanelButtonTemplate")
        btnApply:SetPoint("TOPLEFT", TalentProfiles_main, "TOPLEFT", 10 + 150 + 10, -1)
        btnApply:SetSize(100, 25)
        btnApply:SetText("Apply")
        btnApply:SetScript("OnClick", Handler_ActivateProfile)
        btnApply:Show()
    end
    local btnSave = TalentProfiles_btnSave
    if TalentProfiles_btnSave == nil then
        btnSave = CreateFrame("Button", "TalentProfiles_btnSave", TalentProfiles_main, "UIPanelButtonTemplate")
        btnSave:SetPoint("TOPLEFT", TalentProfiles_main, "TOPLEFT", 10 + 150 + 10 + 100 + 10, -1)
        btnSave:SetSize(100, 25)
        btnSave:SetText("Save")
        btnSave:SetScript("OnClick", Handler_SaveProfile)
        btnSave:Show()
    end
    local btnRemove = TalentProfiles_btnRemove
    if TalentProfiles_btnRemove == nil then
        btnRemove = CreateFrame("Button", "TalentProfiles_btnRemove", TalentProfiles_main, "UIPanelButtonTemplate")
        btnRemove:SetPoint("TOPLEFT", TalentProfiles_main, "TOPLEFT", 10 + 150 + 10 + 100 + 10 + 100 + 10, -1)
        btnRemove:SetSize(100, 25)
        btnRemove:SetText("Remove")
        btnRemove:SetScript("OnClick", Handler_RemoveProfile)
        btnRemove:Show()
    end
end

-------------------- Wow API Events/Hooks -------------------- 

-- Fired when talent the talent frame is toggled
function AddOn:ToggleTalentFrame()
    -- This is raw hooked, so call the origanal function
    self.hooks.ToggleTalentFrame()

    -- Build the frame
    BuildFrame()
    -- Set the visibility of the profile selector to that of the talent frame
    TalentProfiles_main:SetShown(PlayerTalentFrame:IsVisible())
end

-------------------- Ace Events --------------------

function AddOn:OnInitialize()
    -- Load DB
    VerifyDB()

    -- Hook functions
    self:RawHook("ToggleTalentFrame", true)

    self:Print("Loaded")
end
